/**
  **************************************************************************
  * @file     dvp.h
  * @version  v2.0.0
  * @date     2020-11-02
  * @brief    header file of dvp program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __DVP_H
#define __DVP_H

#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "at32f435_437.h"

/** @addtogroup AT32F435_application_note
  * @{
  */
#define LCD_DISPLAY
#define CAMERA_SIZE 240

#define CLIP(X) ((X) > 255 ? 255 : (X) < 0 ? 0 : X)
#define RGB565_RED    0x001F
#define RGB565_GREEN  0x07E0
#define RGB565_BLUE   0xF800

// RGB -> YUV
#define RGB2Y(R, G, B) CLIP(((66 * (R) + 129 * (G) + 25 * (B) + 128) >> 8) + 16)
#define RGB2U(R, G, B) CLIP(((-38 * (R)-74 * (G) + 112 * (B) + 128) >> 8) + 128)
#define RGB2V(R, G, B) CLIP(((112 * (R)-94 * (G)-18 * (B) + 128) >> 8) + 128)

/** @addtogroup 435_qr_decode_by_zbar
  * @{
  */

extern edma_init_type edma_init_struct;
void dvp_config(uint16_t id);
void dvp_dma_init(uint32_t mem0addr,uint16_t memsize);
void dvp_dma_switch_buffer(uint32_t mem0addr, uint16_t memsize);
void xmc_dma_switch_buffer(uint32_t mem0addr, uint16_t memsize);
void xmc_dma_init(uint32_t mem0addr, uint16_t memsize);
void dvp_start(void);
void dvp_stop(void);
void dvp_io_init(void);

void RGB5652YUV(uint16_t source1, uint16_t source2, uint8_t *dest1, uint8_t *dest2);
void ConvertRGB2Y8(uint16_t *sourceLcdBuffer, uint8_t *destLcdBuffer);
void ConvertY8toRGB565(uint8_t *sourceLcdBuffer, uint16_t *destLcdBuffer);

/**
  * @}
  */

/**
  * @}
  */

#ifdef __cplusplus
}
#endif

#endif /*__DVP_H */
